////
//// A Freebasic IrrKlang Interface Test
//// F.Dodd (2006)
////
//// Pascal conversion by Johannes Stein (2009)
//// http://www.freeze-dev.de ; info@freeze-dev.de
////
//// This is a pure test of creating a wrapper DLL for IrrKlang simplifying its
//// C++ architecture and providing a basic but functional Audio SDK for languages
//// that are not object oriented. At the moment this provides very few calls and
//// is merely a test for implementing such an interface.
////
//// This source uses Irrlicht definitions created by the Irrlicht authors
////

//// ////////////////////////////////////////////////////////////////////////////

unit IrrKlangWrapper;

{$IFNDEF WIN32}
  {$LINKLIB libIrrKlang}
{$ENDIF}
  
interface

//// ////////////////////////////////////////////////////////////////////////////
//// Constant Definitions
const
  {$IFDEF UNIX}
    {$IFDEF DARWIN}
	  DLLNAME = 'libIrrKlangWrapper.dylib';
	{$ELSE}
	  DLLNAME = 'libIrrKlangWrapper.so.0.1';
	{$ENDIF}
  {$ELSE}
	DLLNAME = 'IrrKlangWrapper.dll';
  {$ENDIF}

  IRRKLANGFALSE = 0;
  IRRKLANGTRUE = 1;
  


type
//// ////////////////////////////////////////////////////////////////////////////
//// Enumerated Type Definitions

ESTREAMMODE = (
    ESMAUTODETECT = 0,
    ESMSTREAMING,
    ESMNOSTREAMING,
    ESMFORCE32BIT = $fffffff
);


//// ////////////////////////////////////////////////////////////////////////////
//// Type Definitions

irrsound = PCardinal;


//// ////////////////////////////////////////////////////////////////////////////
//// System Functions

// only one irrklang device object is supported by this wrapper. this call creates
// the device object that is exposed to FreeBasic
procedure IrrKlangStart(); cdecl; external DLLNAME;

// Set the global sound volume
procedure IrrKlangSetSoundVolume(volume: single); cdecl; external DLLNAME;

// Play a 2D sound
function IrrKlangPlay2D(
     filename: PChar;
     playLooped: Cardinal = IRRKLANGFALSE;
     startPaused: Cardinal = IRRKLANGFALSE;
     track: Cardinal = IRRKLANGFALSE;
     streamMode: ESTREAMMODE = ESMAUTODETECT; 
     enableSoundEffects: Cardinal = IRRKLANGFALSE): irrsound; cdecl; external DLLNAME;

// Play a 3D sound
function IrrKlangPlay3D( 
     filename: PChar; 
     posX, posY, posZ: single; 
     playLooped: Cardinal = IRRKLANGFALSE;
     startPaused: Cardinal = IRRKLANGFALSE; 
     track: Cardinal = IRRKLANGFALSE;
     streamMode: ESTREAMMODE = ESMAUTODETECT; 
     enableSoundEffects: Cardinal = IRRKLANGFALSE): irrsound; cdecl; external DLLNAME;


// Pause all sounds
procedure IrrKlangSetAllSoundsPaused(volume: integer); cdecl; external DLLNAME;

// Stop all sounds
procedure IrrKlangStopAllSounds(); cdecl; external DLLNAME;

// Remove a specific sound file from memory
procedure IrrRemoveSoundSource(filename: PChar); cdecl; external DLLNAME;

// Stop all sounds and remove all sound files from memory
procedure IrrKlangRemoveAllSoundSources(); cdecl; external DLLNAME;

// Set the position of the listener for 3D sounds
procedure IrrKlangSetListenerPosition( 
     posX, posY, posZ, lookX, lookY, lookZ: single;
     velocityX: single = 0;
     velocityY: single = 0; 
     velocityZ: single = 0; 
     upX: single = 0;
     upY: single = 1; 
     upZ: single = 0); cdecl; external DLLNAME;

// Stop the sound engine
procedure IrrKlangStop(); cdecl; external DLLNAME;


//// ////////////////////////////////////////////////////////////////////////////
//// Sound Functions

// Release the sound
procedure IrrReleaseSound(sound: irrsound); cdecl; external DLLNAME;

// Get the length of the sound in milliseconds
function IrrKlangGetPlayLength(sound: irrsound): Cardinal; cdecl; external DLLNAME;

// Get the position of play in the sound in milliseconds
function IrrKlangGetPlayPosition(sound: irrsound): Cardinal; cdecl; external DLLNAME;

// Determine if a sound has finished playing
function IrrKlangIsFinished(sound: irrsound): Cardinal; cdecl; external DLLNAME;

// Pause and unpause the sound
procedure IrrKlangSetIsPaused(sound: irrsound; pause: Cardinal); cdecl; external DLLNAME;

// Set the playback speed of the sound
procedure IrrKlangSetPlaybackSpeed(sound: irrsound; speed: single); cdecl; external DLLNAME;

// Set the playback position of the sound
procedure IrrKlangSetPlayPosition(sound: irrsound; position: Cardinal); cdecl; external DLLNAME;

// Set the position of the sound in 3D space
procedure IrrKlangSetPosition(sound: irrsound; posX, posY, posZ: single); cdecl; external DLLNAME;

// Set the volume of the sound
procedure IrrKlangSetVolume(sound: irrsound; volume: single); cdecl; external DLLNAME;

// Set the minimum distance at which the sound can be heard
procedure IrrKlangSetMinDistance(sound: irrsound; distance: single); cdecl; external DLLNAME;

// Stop the sound playing
procedure IrrKlangStopSound(sound: irrsound); cdecl; external DLLNAME;


implementation

end.
